/**
 * Maling.java - "Programmering i Java", 4.utgave - 2009-07-01
 *
 * Klassen Maling tilbyr metoder for beregning av materialbehov og pris.
 * Attributter: Malingsnavn (identifikator), pris, antall strk og dekke-evne.
 */

// Lsning oppgave i kap 11
class Maling {
  private static final double GRENSE = 0.02;  // grense for  kjpe 0.5 liter maling til
  private final String navn;  // identifiserer malingen
  private final double pris;  // pr.liter
  private final int antStrk;
  private final double antKvmPrLiter;

  public Maling(String navn, double pris, int antStrk, double antKvmPrLiter) {
    if (navn == null || navn.trim().equals("")) {
      throw new IllegalArgumentException("Malingsnavn m vre oppgitt.");
    }
    if (pris <= 0.0 || antStrk <= 0 || antKvmPrLiter <= 0.0) {
      throw new IllegalArgumentException(
                "Bde pris, ant. strk og dekkevne (kvm/l) m vre positive tall.\n" +
                "Inndata til konstruktren var: pris = " + pris +
                ", ant. strk = " + antStrk + ", dekkevne (kvm/l) = " + antKvmPrLiter);
    }
    this.navn = navn;
    this.antStrk = antStrk;
    this.antKvmPrLiter = antKvmPrLiter;
    this.pris = pris;
  }

  public String getNavn() {
    return navn;
  }

  public double getPrisPrLiter() {
    return pris;
  }

  public int getAntStrk() {
    return antStrk;
  }

  public double getAntKvmPrLiter() {
    return antKvmPrLiter;
  }

  /**
   * Metoden finner antall liter maling som trengs,
   * behovet rundes oppover til nrmeste halve liter.
   */
  public double beregnAntLiter(Flate enFlate) {
    double areal = enFlate.beregnAreal();
    double antLiter = areal * antStrk / antKvmPrLiter;
    int heltAntallLitere = (int) antLiter;
    double overLiteren = antLiter - heltAntallLitere;
    if (overLiteren >= 0.5 + GRENSE) {
      return heltAntallLitere + 1.0;
    } else {
      if (overLiteren >= GRENSE) {
        return heltAntallLitere + 0.5;
      } else {
        return heltAntallLitere;
      }
    }
  }

  /**
   * Metoden regner ut hva malingen til en bestemt flate koster.
   */
  public double beregnTotalpris(Flate enFlate) {
    return beregnAntLiter(enFlate) * pris;
  }

  /**
   * To objekter er like dersom navnet er det samme.
   */
  public boolean equals(Object obj) {
    if (!(obj instanceof Maling)) {
      return false;
    }
    if (this == obj) {
      return true;
    }
    Maling m = (Maling) obj;
    return (navn.equals(m.getNavn()));
  }

  public String toString() {
    java.util.Formatter f = new java.util.Formatter();
    f.format("Maling: %s, pris kr  %.2f  pr. liter, ant. strk: %d, ant. kvm/l %.2f",
                 navn, pris, antStrk, antKvmPrLiter);
    return f.toString();
  }
}